@@ -204,8 +204,8 @@ gpa_printer_new_from_tree (xmlNodePtr tree)
 	xmlNodePtr node;
 	GPANode *settings, *printer, *model;
 	xmlChar *name, *id, *version;
-	const gchar *lang = NULL;
-
+	const char **syslangs = NULL;
+	
 	g_return_val_if_fail (tree != NULL, NULL);
 	g_return_val_if_fail (tree->name != NULL, NULL);
 
@@ -229,45 +229,46 @@ gpa_printer_new_from_tree (xmlNodePtr tree)
 			   "for printer \"%s\"", version, id);
 		goto gpa_printer_new_from_tree_error;
 	}
-
-#ifdef HAVE_LC_MESSAGES
-	lang = setlocale (LC_MESSAGES, NULL);
+	
+#if GLIB_CHECK_VERSION (2, 6, 0)
+	syslangs = g_get_language_names ();
 #endif
-	node = tree->xmlChildrenNode;
-	for (; node != NULL; node = node->next) {
-
-		if (strcmp (node->name, "Name") == 0) {
-			xmlChar *node_lang;
-			node_lang = xmlNodeGetLang (node);
-			if (node_lang && lang && !strcmp (lang, node_lang)) {
-				my_xmlFree (name);
+	
+	for (node = tree->xmlChildrenNode; node != NULL; node = node->next) {
+		if (!strcmp (node->name, "Name")) {
+			if (syslangs != NULL) {
+				xmlChar *xmllang = xmlNodeGetLang (node);
+				int i;
+				
+				for (i = 0; syslangs[i] && xmllang; i++) {
+					if (!strcmp (syslangs[i], xmllang)) {
+						xmlFree (name);
+						name = xmlNodeGetContent (node);
+						break;
+					}
+				}
+				
+				if (!xmllang)
+					name = xmlNodeGetContent (node);
+				else
+					xmlFree (xmllang);
+			} else if (!name) {
 				name = xmlNodeGetContent (node);
 			}
-			if (node_lang == NULL && name == NULL) {
-				name = xmlNodeGetContent (node);
-			}
-			xmlFree (node_lang);
-			continue;
-		}
-
-		if (strcmp (node->name, "Model") == 0) {
+		} else if (!strcmp (node->name, "Model")) {
 			xmlChar *model_id = xmlNodeGetContent (node);
+			
 			model = gpa_model_get_by_id (model_id, FALSE);
-			my_xmlFree (model_id);
-			continue;
-		}
-		
-		if (strcmp (node->name, "Settings") == 0) {
+			xmlFree (model_id);
+		} else if (!strcmp (node->name, "Settings")) {
 			/* We don't support multiple settings per printer yet */
 			g_assert (settings == NULL);
-			if (!model) {
+			if (model != NULL) {
+				settings = gpa_settings_new_from_model_and_tree (model, node);
+			} else {
 				g_warning ("<Model> node should come before <Settings> (\"%s\")", id);
-				continue;
 			}
-			settings = gpa_settings_new_from_model_and_tree (model, node);
-			continue;
 		}
-
 	}
 
 	if (!name || !name[0]) {
@@ -286,9 +287,9 @@ gpa_printer_new_from_tree (xmlNodePtr tree)
 	printer = gpa_printer_new (id, name, GPA_MODEL (model), GPA_SETTINGS (settings));
 
 gpa_printer_new_from_tree_error:
-	my_xmlFree (name);
-	my_xmlFree (id);
-	my_xmlFree (version);
+	xmlFree (name);
+	xmlFree (id);
+	xmlFree (version);
 
 	if (!printer) {
 		my_gpa_node_unref (settings);
